/*
 * Copyright (C) 2006 Christian Hohnstaedt <chohnstaedt@innominate.com>
 *
 * This file is released under the GPLv2
 */

#ifndef NPE_DEVICE_H
#define NPE_DEVICE_H

#include <linux/miscdevice.h>
#include <asm/hardware.h>

struct npe_info {
	struct resource *res;
	void __iomem *addr;
	struct npe_plat_data *plat;
	u8 img_info[4];
	u32 exec_count;
	u32 ctx_reg2;
};


static inline void npe_reg_write(struct npe_info *npe, u32 reg, u32 val)
{
	*(volatile u32*)((u8*)(npe->addr) + reg) = val;
}

static inline u32 npe_reg_read(struct npe_info *npe, u32 reg)
{
	return *(volatile u32*)((u8*)(npe->addr) + reg);
}

static inline u32 npe_status(struct npe_info *npe)
{
	return npe_reg_read(npe, IX_NPEDL_REG_OFFSET_EXCTL);
}

/* ixNpeDlNpeMgrCommandIssue */
static inline void npe_write_exctl(struct npe_info *npe, u32 cmd)
{
	npe_reg_write(npe, IX_NPEDL_REG_OFFSET_EXCTL, cmd);
}
/* ixNpeDlNpeMgrWriteCommandIssue */
static inline void
npe_write_cmd(struct npe_info *npe, u32 addr, u32 data, int cmd)
{
	npe_reg_write(npe, IX_NPEDL_REG_OFFSET_EXDATA, data);
	npe_reg_write(npe, IX_NPEDL_REG_OFFSET_EXAD, addr);
	npe_reg_write(npe, IX_NPEDL_REG_OFFSET_EXCTL, cmd);
}
/* ixNpeDlNpeMgrReadCommandIssue */
static inline u32
npe_read_cmd(struct npe_info *npe, u32 addr, int cmd)
{
	npe_reg_write(npe, IX_NPEDL_REG_OFFSET_EXAD, addr);
	npe_reg_write(npe, IX_NPEDL_REG_OFFSET_EXCTL, cmd);
	/* Intel reads the data twice - so do we... */
	npe_reg_read(npe, IX_NPEDL_REG_OFFSET_EXDATA);
	return npe_reg_read(npe, IX_NPEDL_REG_OFFSET_EXDATA);
}

/* ixNpeDlNpeMgrExecAccRegWrite */
static inline void npe_write_ecs_reg(struct npe_info *npe, u32 addr, u32 data)
{
	npe_write_cmd(npe, addr, data, IX_NPEDL_EXCTL_CMD_WR_ECS_REG);
}
/* ixNpeDlNpeMgrExecAccRegRead */
static inline u32 npe_read_ecs_reg(struct npe_info *npe, u32 addr)
{
	return npe_read_cmd(npe, addr, IX_NPEDL_EXCTL_CMD_RD_ECS_REG);
}

extern struct device *get_npe_by_id(int id);

/* NPE Messages */
extern int
npe_mh_setportaddr(struct npe_info *npe, struct mac_plat_info *mp, u8 *macaddr);
extern int
npe_mh_disable_firewall(struct npe_info *npe, struct mac_plat_info *mp);
extern int
npe_mh_set_rxqid(struct npe_info *npe, struct mac_plat_info *mp, int qid);
extern int
npe_mh_npe_loopback_mode(struct npe_info *npe, struct mac_plat_info *mp, int enable);

#endif
